#!/usr/bin/perl
#
# The script installs Silicom cards information into the PCMCIA package config
# file. Our cards information in the file is delimited by signature that
# allows insertion/replacement according to the cardinfo version (the first
# line of the cards information file).
#
# Script can receive one parameter - the base dir for the PCMCIA package.
# The default base dir is /etc/pcmcia.
# Returns 0 - success, error otherwise.
#
# Written by Igor Royzis (c) 1998
# Date: 14-dec-98
# Last update: 15-dec-98
#

$pconfigfile="config";           # PCMCIA config file name


$basedir= shift;
if( length($basedir) == 0)  {
   $basedir="/etc/pcmcia";       # Default base dir value
}

$configfile=join("",$basedir,"/",$pconfigfile);
$confignew=join("",$basedir,"/","new.config");       
$cardinfo="silicom.crd";

$signature_start="sil_ver";
$signature_end="sil_end";
$idone=0;
$notchanged=0;

local *FH;
local *FO;
local *FC;

#
sub InsOurConfigData
{
   print FO "# $signature_start$version\n";
   while (<FC>)  {
     print FO $_;
   }
   print FO "# $signature_end\n";
}


# Make filename for writting
$confignewf=join("",'>',$confignew);

# Open the card info file
if(!open(FC, $cardinfo))  {
  print "*******************************************************\n";
  print "Can not open the card information file.\n";
  print "*******************************************************\n";
  exit 3;
}
# Read the version from the card info file, remove end of line
$version = <FC>;
chomp($version);

# Open new file for writting
if(!open(FO, $confignewf))  {
  print "*******************************************************\n";
  print "Can not open file for writing. Please check permissions\n";
  print "*******************************************************\n";
  exit 2;
}

if(open(FH, $configfile))  {
  while(<FH>)  {
    # Find string
    $j=/$signature_start([0-9A-Fa-f]+)/i;
    if($j == "")  {
        # Our signature not found, find the last line of the input file
        # I assume that the last line(s) of the config file are includes
        $j=/source .+/;
        if($j == "")  {
          # Signature not found, copy the input file to the output
          print FO $_;
        }
        else  {
          # Include directive found, insert our data before other includes
          $tmpstr = $_;                # $_ will be overwritten
          InsOurConfigData();
          print FO $tmpstr;
          # Copy out the rest of file
          while(<FH>)  {
            print FO $_;
          }
          $idone=1;
          last;
        }
    }
    else  {   
        # Signature found     
        if($1 >= $version)  {
          print "Configuration information is already up-to-date, no changes were made\n";
          print FO $_;
          # Copy out the rest of the file
          while(<FH>)  {
            print FO $_;
          }
          $idone=1;
          $notchanged=1;
          last;
        }
        else  {
          print "Configuration information has version $1, updating with version $version\n";
          InsOurConfigData();
          # Skip the input data till signature end
          while(<FH>)  {
             $j=/$signature_end/i;
             if($j != "") { last };
          }
          # Copy out the rest of file
          while(<FH>)  {
           print FO $_;
          };
          $idone=1;
          last;
        }
    }
  }
  if($idone == 0)  {
    # Still our data not there? Append it to the end of file
    print FO "\n";
    InsOurConfigData();
  }
  close FO;
  close FH;
  close FC;
 
  if( $notchanged == 0) {
     $backfile=join("",$configfile,'.bak');
     rename $configfile, $backfile;         # Make backup copy
     rename $confignew, $configfile;        # Rename new file to config one
  }
  else  {
     unlink $confignew;                     # No changes, remove new file
  }
  exit 0;
}
else  {
  print "***********************************************\n";
  print "Configuration file $configfile not found\n";
  print "***********************************************\n";
  exit 1;
}